/*
 * info.c - routines for printing info, updating image keywords, etc
 *
 * $Id: info.c,v 1.2 2007/09/24 04:17:56 johns Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "util.h"
#include "serial.h"
#include "gps.h"
#include "info.h"

static void donthogcpu(void) {
  rt_sleep(10); /* don't hog the CPU while in polling loops */
}

int print_gps_status(GPSHandle gps) {
  char datebuf[1024];
  char timebuf[1024];
  char latbuf[1024];
  char longbuf[1024];
  int fix=0, numsats=0; 
  float dop=99;

  if (!gps_get_fixquality(gps, &fix) && (fix > 0)) {
    gps_get_numsats(gps, &numsats);
    gps_get_dop(gps, &dop); 
    gps_sprint_date_utc(gps, datebuf);
    gps_sprint_time_utc(gps, timebuf);
    gps_sprint_lat(gps, latbuf);
    gps_sprint_long(gps, longbuf);
    printf("GPS fix=%d sats=%d dop=%3.1f lat: %s long: %s  date: %s %s\n",
           fix, numsats, dop, latbuf, longbuf, timebuf, datebuf);
  } else {
    printf("GPS unable to get a fix, continuing without lat/long data...\n");
    return -1;
  }

  return 0;
}

int get_gps_fix(GPSHandle gps, int dataneeded) {
  int gotdata;
  float time, gpscommwait=2.0, gpsfixwait=6.0;
  rt_timerhandle timer;
  msgtimer msgt;
  int done;

  timer = rt_timer_create();
  rt_timer_start(timer);

  gps_clear_data(gps); /* only report current data, not old stuff */
  gotdata = 0;
  msg_timer_init(&msgt, 0.2);
  done = 0;
  time = 0;
  while (!done) { 
    int rc, havedata;

    rc = gps_get_status(gps);
    if (rc > 0) {
      gotdata += rc;
    } else if (rc < 0) {
      printf("GPS error, aborting\n");
      return -1;
    }

    gps_have_data(gps, &havedata);
    if (!(havedata & GPS_COMM)) {
      if (msg_timer_timeout(&msgt)) {
        printf("Listening for GPS NMEA report..., %3.1f more sec...\r", gpscommwait - time);
        fflush(stdout);
      }
    } else { 
      if (msg_timer_timeout(&msgt)) {
        printf("Acquiring GPS position fix, %3.1f more sec...      \r", gpsfixwait - time);
        fflush(stdout);
      }
    }

    if ((havedata & dataneeded) == dataneeded) {
      done = 1; 
      continue; /* we've got what we need, finish early */
    } else {
      donthogcpu();
    }

    time = rt_timer_timenow(timer);
    if (!(havedata & GPS_COMM) && time > gpscommwait) { 
      done = 1;
    }

    if (time > gpsfixwait)
      done = 1;
  }

  printf("\r");

  rt_timer_destroy(timer);

  return 0;
}

